@@ -3,7 +3,7 @@ import logging
 import struct
 from typing import Optional
 
-from pymodbus.exceptions import ConnectionException, ModbusException
+from pymodbus.exceptions import ModbusException
 from pymodbus.pdu import ExceptionResponse
 import voluptuous as vol
 
@@ -21,30 +21,31 @@ from homeassistant.const import (
 )
 import homeassistant.helpers.config_validation as cv
 
-from . import CONF_HUB, DEFAULT_HUB, DOMAIN as MODBUS_DOMAIN
+from .const import (
+    CALL_TYPE_REGISTER_HOLDING,
+    CALL_TYPE_REGISTER_INPUT,
+    CONF_CURRENT_TEMP,
+    CONF_CURRENT_TEMP_REGISTER_TYPE,
+    CONF_DATA_COUNT,
+    CONF_DATA_TYPE,
+    CONF_HUB,
+    CONF_MAX_TEMP,
+    CONF_MIN_TEMP,
+    CONF_OFFSET,
+    CONF_PRECISION,
+    CONF_SCALE,
+    CONF_STEP,
+    CONF_TARGET_TEMP,
+    CONF_UNIT,
+    DATA_TYPE_FLOAT,
+    DATA_TYPE_INT,
+    DATA_TYPE_UINT,
+    DEFAULT_HUB,
+    MODBUS_DOMAIN,
+)
 
 _LOGGER = logging.getLogger(__name__)
 
-CONF_TARGET_TEMP = "target_temp_register"
-CONF_CURRENT_TEMP = "current_temp_register"
-CONF_CURRENT_TEMP_REGISTER_TYPE = "current_temp_register_type"
-CONF_DATA_TYPE = "data_type"
-CONF_COUNT = "data_count"
-CONF_PRECISION = "precision"
-CONF_SCALE = "scale"
-CONF_OFFSET = "offset"
-CONF_UNIT = "temperature_unit"
-DATA_TYPE_INT = "int"
-DATA_TYPE_UINT = "uint"
-DATA_TYPE_FLOAT = "float"
-CONF_MAX_TEMP = "max_temp"
-CONF_MIN_TEMP = "min_temp"
-CONF_STEP = "temp_step"
-SUPPORT_FLAGS = SUPPORT_TARGET_TEMPERATURE
-HVAC_MODES = [HVAC_MODE_AUTO]
-
-DEFAULT_REGISTER_TYPE_HOLDING = "holding"
-DEFAULT_REGISTER_TYPE_INPUT = "input"
 
 PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
     {
@@ -52,10 +53,10 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
         vol.Required(CONF_NAME): cv.string,
         vol.Required(CONF_SLAVE): cv.positive_int,
         vol.Required(CONF_TARGET_TEMP): cv.positive_int,
-        vol.Optional(CONF_COUNT, default=2): cv.positive_int,
+        vol.Optional(CONF_DATA_COUNT, default=2): cv.positive_int,
         vol.Optional(
-            CONF_CURRENT_TEMP_REGISTER_TYPE, default=DEFAULT_REGISTER_TYPE_HOLDING
-        ): vol.In([DEFAULT_REGISTER_TYPE_HOLDING, DEFAULT_REGISTER_TYPE_INPUT]),
+            CONF_CURRENT_TEMP_REGISTER_TYPE, default=CALL_TYPE_REGISTER_HOLDING
+        ): vol.In([CALL_TYPE_REGISTER_HOLDING, CALL_TYPE_REGISTER_INPUT]),
         vol.Optional(CONF_DATA_TYPE, default=DATA_TYPE_FLOAT): vol.In(
             [DATA_TYPE_INT, DATA_TYPE_UINT, DATA_TYPE_FLOAT]
         ),
@@ -71,7 +72,7 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
 )
 
 
-def setup_platform(hass, config, add_entities, discovery_info=None):
+async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
     """Set up the Modbus Thermostat Platform."""
     name = config[CONF_NAME]
     modbus_slave = config[CONF_SLAVE]
@@ -79,7 +80,7 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
     current_temp_register = config[CONF_CURRENT_TEMP]
     current_temp_register_type = config[CONF_CURRENT_TEMP_REGISTER_TYPE]
     data_type = config[CONF_DATA_TYPE]
-    count = config[CONF_COUNT]
+    count = config[CONF_DATA_COUNT]
     precision = config[CONF_PRECISION]
     scale = config[CONF_SCALE]
     offset = config[CONF_OFFSET]
@@ -90,7 +91,7 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
     hub_name = config[CONF_HUB]
     hub = hass.data[MODBUS_DOMAIN][hub_name]
 
-    add_entities(
+    async_add_entities(
         [
             ModbusThermostat(
                 hub,
@@ -162,19 +163,19 @@ class ModbusThermostat(ClimateDevice):
             DATA_TYPE_FLOAT: {1: "e", 2: "f", 4: "d"},
         }
 
-        self._structure = ">{}".format(data_types[self._data_type][self._count])
+        self._structure = f">{data_types[self._data_type][self._count]}"
 
     @property
     def supported_features(self):
         """Return the list of supported features."""
-        return SUPPORT_FLAGS
+        return SUPPORT_TARGET_TEMPERATURE
 
-    def update(self):
+    async def async_update(self):
         """Update Target & Current Temperature."""
-        self._target_temperature = self._read_register(
-            DEFAULT_REGISTER_TYPE_HOLDING, self._target_temperature_register
+        self._target_temperature = await self._read_register(
+            CALL_TYPE_REGISTER_HOLDING, self._target_temperature_register
         )
-        self._current_temperature = self._read_register(
+        self._current_temperature = await self._read_register(
             self._current_temperature_register_type, self._current_temperature_register
         )
 
@@ -186,7 +187,7 @@ class ModbusThermostat(ClimateDevice):
     @property
     def hvac_modes(self):
         """Return the possible HVAC modes."""
-        return HVAC_MODES
+        return [HVAC_MODE_AUTO]
 
     @property
     def name(self):
@@ -223,7 +224,7 @@ class ModbusThermostat(ClimateDevice):
         """Return the supported step of target temperature."""
         return self._temp_step
 
-    def set_temperature(self, **kwargs):
+    async def set_temperature(self, **kwargs):
         """Set new target temperature."""
         target_temperature = int(
             (kwargs.get(ATTR_TEMPERATURE) - self._offset) / self._scale
@@ -232,30 +233,28 @@ class ModbusThermostat(ClimateDevice):
             return
         byte_string = struct.pack(self._structure, target_temperature)
         register_value = struct.unpack(">h", byte_string[0:2])[0]
-        self._write_register(self._target_temperature_register, register_value)
+        await self._write_register(self._target_temperature_register, register_value)
 
     @property
     def available(self) -> bool:
         """Return True if entity is available."""
         return self._available
 
-    def _read_register(self, register_type, register) -> Optional[float]:
+    async def _read_register(self, register_type, register) -> Optional[float]:
         """Read register using the Modbus hub slave."""
-        try:
-            if register_type == DEFAULT_REGISTER_TYPE_INPUT:
-                result = self._hub.read_input_registers(
-                    self._slave, register, self._count
-                )
-            else:
-                result = self._hub.read_holding_registers(
-                    self._slave, register, self._count
-                )
-        except ConnectionException:
-            self._set_unavailable(register)
+        if register_type == CALL_TYPE_REGISTER_INPUT:
+            result = await self._hub.read_input_registers(
+                self._slave, register, self._count
+            )
+        else:
+            result = await self._hub.read_holding_registers(
+                self._slave, register, self._count
+            )
+        if result is None:
+            self._available = False
             return
-
         if isinstance(result, (ModbusException, ExceptionResponse)):
-            self._set_unavailable(register)
+            self._available = False
             return
 
         byte_string = b"".join(
@@ -270,25 +269,7 @@ class ModbusThermostat(ClimateDevice):
 
         return register_value
 
-    def _write_register(self, register, value):
+    async def _write_register(self, register, value):
         """Write holding register using the Modbus hub slave."""
-        try:
-            self._hub.write_registers(self._slave, register, [value, 0])
-        except ConnectionException:
-            self._set_unavailable(register)
-            return
-
+        await self._hub.write_registers(self._slave, register, [value, 0])
         self._available = True
-
-    def _set_unavailable(self, register):
-        """Set unavailable state and log it as an error."""
-        if not self._available:
-            return
-
-        _LOGGER.error(
-            "No response from hub %s, slave %s, register %s",
-            self._hub.name,
-            self._slave,
-            register,
-        )
-        self._available = False
