@@ -2,7 +2,7 @@
 import logging
 from typing import Optional
 
-from pymodbus.exceptions import ConnectionException, ModbusException
+from pymodbus.exceptions import ModbusException
 from pymodbus.pdu import ExceptionResponse
 import voluptuous as vol
 
@@ -18,22 +18,25 @@ from homeassistant.helpers import config_validation as cv
 from homeassistant.helpers.entity import ToggleEntity
 from homeassistant.helpers.restore_state import RestoreEntity
 
-from . import CONF_HUB, DEFAULT_HUB, DOMAIN as MODBUS_DOMAIN
+from .const import (
+    CALL_TYPE_COIL,
+    CALL_TYPE_REGISTER_HOLDING,
+    CALL_TYPE_REGISTER_INPUT,
+    CONF_COILS,
+    CONF_HUB,
+    CONF_REGISTER,
+    CONF_REGISTER_TYPE,
+    CONF_REGISTERS,
+    CONF_STATE_OFF,
+    CONF_STATE_ON,
+    CONF_VERIFY_REGISTER,
+    CONF_VERIFY_STATE,
+    DEFAULT_HUB,
+    MODBUS_DOMAIN,
+)
 
 _LOGGER = logging.getLogger(__name__)
 
-CONF_COIL = "coil"
-CONF_COILS = "coils"
-CONF_REGISTER = "register"
-CONF_REGISTER_TYPE = "register_type"
-CONF_REGISTERS = "registers"
-CONF_STATE_OFF = "state_off"
-CONF_STATE_ON = "state_on"
-CONF_VERIFY_REGISTER = "verify_register"
-CONF_VERIFY_STATE = "verify_state"
-
-DEFAULT_REGISTER_TYPE_HOLDING = "holding"
-DEFAULT_REGISTER_TYPE_INPUT = "input"
 
 REGISTERS_SCHEMA = vol.Schema(
     {
@@ -42,8 +45,8 @@ REGISTERS_SCHEMA = vol.Schema(
         vol.Required(CONF_NAME): cv.string,
         vol.Required(CONF_REGISTER): cv.positive_int,
         vol.Optional(CONF_HUB, default=DEFAULT_HUB): cv.string,
-        vol.Optional(CONF_REGISTER_TYPE, default=DEFAULT_REGISTER_TYPE_HOLDING): vol.In(
-            [DEFAULT_REGISTER_TYPE_HOLDING, DEFAULT_REGISTER_TYPE_INPUT]
+        vol.Optional(CONF_REGISTER_TYPE, default=CALL_TYPE_REGISTER_HOLDING): vol.In(
+            [CALL_TYPE_REGISTER_HOLDING, CALL_TYPE_REGISTER_INPUT]
         ),
         vol.Optional(CONF_SLAVE): cv.positive_int,
         vol.Optional(CONF_STATE_OFF): cv.positive_int,
@@ -55,7 +58,7 @@ REGISTERS_SCHEMA = vol.Schema(
 
 COILS_SCHEMA = vol.Schema(
     {
-        vol.Required(CONF_COIL): cv.positive_int,
+        vol.Required(CALL_TYPE_COIL): cv.positive_int,
         vol.Required(CONF_NAME): cv.string,
         vol.Required(CONF_SLAVE): cv.positive_int,
         vol.Optional(CONF_HUB, default=DEFAULT_HUB): cv.string,
@@ -73,7 +76,7 @@ PLATFORM_SCHEMA = vol.All(
 )
 
 
-def setup_platform(hass, config, add_entities, discovery_info=None):
+async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
     """Read configuration and create Modbus devices."""
     switches = []
     if CONF_COILS in config:
@@ -82,7 +85,7 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
             hub = hass.data[MODBUS_DOMAIN][hub_name]
             switches.append(
                 ModbusCoilSwitch(
-                    hub, coil[CONF_NAME], coil[CONF_SLAVE], coil[CONF_COIL]
+                    hub, coil[CONF_NAME], coil[CONF_SLAVE], coil[CALL_TYPE_COIL]
                 )
             )
     if CONF_REGISTERS in config:
@@ -106,7 +109,7 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
                 )
             )
 
-    add_entities(switches)
+    async_add_entities(switches)
 
 
 class ModbusCoilSwitch(ToggleEntity, RestoreEntity):
@@ -143,28 +146,26 @@ class ModbusCoilSwitch(ToggleEntity, RestoreEntity):
         """Return True if entity is available."""
         return self._available
 
-    def turn_on(self, **kwargs):
+    async def turn_on(self, **kwargs):
         """Set switch on."""
-        self._write_coil(self._coil, True)
+        await self._write_coil(self._coil, True)
 
-    def turn_off(self, **kwargs):
+    async def turn_off(self, **kwargs):
         """Set switch off."""
-        self._write_coil(self._coil, False)
+        await self._write_coil(self._coil, False)
 
-    def update(self):
+    async def async_update(self):
         """Update the state of the switch."""
-        self._is_on = self._read_coil(self._coil)
+        self._is_on = await self._read_coil(self._coil)
 
-    def _read_coil(self, coil) -> Optional[bool]:
+    async def _read_coil(self, coil) -> Optional[bool]:
         """Read coil using the Modbus hub slave."""
-        try:
-            result = self._hub.read_coils(self._slave, coil, 1)
-        except ConnectionException:
-            self._set_unavailable()
+        result = await self._hub.read_coils(self._slave, coil, 1)
+        if result is None:
+            self._available = False
             return
-
         if isinstance(result, (ModbusException, ExceptionResponse)):
-            self._set_unavailable()
+            self._available = False
             return
 
         value = bool(result.bits[0])
@@ -172,29 +173,11 @@ class ModbusCoilSwitch(ToggleEntity, RestoreEntity):
 
         return value
 
-    def _write_coil(self, coil, value):
+    async def _write_coil(self, coil, value):
         """Write coil using the Modbus hub slave."""
-        try:
-            self._hub.write_coil(self._slave, coil, value)
-        except ConnectionException:
-            self._set_unavailable()
-            return
-
+        await self._hub.write_coil(self._slave, coil, value)
         self._available = True
 
-    def _set_unavailable(self):
-        """Set unavailable state and log it as an error."""
-        if not self._available:
-            return
-
-        _LOGGER.error(
-            "No response from hub %s, slave %s, coil %s",
-            self._hub.name,
-            self._slave,
-            self._coil,
-        )
-        self._available = False
-
 
 class ModbusRegisterSwitch(ModbusCoilSwitch):
     """Representation of a Modbus register switch."""
@@ -238,21 +221,21 @@ class ModbusRegisterSwitch(ModbusCoilSwitch):
 
         self._is_on = None
 
-    def turn_on(self, **kwargs):
+    async def turn_on(self, **kwargs):
         """Set switch on."""
 
         # Only holding register is writable
-        if self._register_type == DEFAULT_REGISTER_TYPE_HOLDING:
-            self._write_register(self._command_on)
+        if self._register_type == CALL_TYPE_REGISTER_HOLDING:
+            await self._write_register(self._command_on)
             if not self._verify_state:
                 self._is_on = True
 
-    def turn_off(self, **kwargs):
+    async def turn_off(self, **kwargs):
         """Set switch off."""
 
         # Only holding register is writable
-        if self._register_type == DEFAULT_REGISTER_TYPE_HOLDING:
-            self._write_register(self._command_off)
+        if self._register_type == CALL_TYPE_REGISTER_HOLDING:
+            await self._write_register(self._command_off)
             if not self._verify_state:
                 self._is_on = False
 
@@ -261,12 +244,12 @@ class ModbusRegisterSwitch(ModbusCoilSwitch):
         """Return True if entity is available."""
         return self._available
 
-    def update(self):
+    async def async_update(self):
         """Update the state of the switch."""
         if not self._verify_state:
             return
 
-        value = self._read_register()
+        value = await self._read_register()
         if value == self._state_on:
             self._is_on = True
         elif value == self._state_off:
@@ -280,20 +263,20 @@ class ModbusRegisterSwitch(ModbusCoilSwitch):
                 value,
             )
 
-    def _read_register(self) -> Optional[int]:
-        try:
-            if self._register_type == DEFAULT_REGISTER_TYPE_INPUT:
-                result = self._hub.read_input_registers(self._slave, self._register, 1)
-            else:
-                result = self._hub.read_holding_registers(
-                    self._slave, self._register, 1
-                )
-        except ConnectionException:
-            self._set_unavailable()
+    async def _read_register(self) -> Optional[int]:
+        if self._register_type == CALL_TYPE_REGISTER_INPUT:
+            result = await self._hub.read_input_registers(
+                self._slave, self._register, 1
+            )
+        else:
+            result = await self._hub.read_holding_registers(
+                self._slave, self._register, 1
+            )
+        if result is None:
+            self._available = False
             return
-
         if isinstance(result, (ModbusException, ExceptionResponse)):
-            self._set_unavailable()
+            self._available = False
             return
 
         value = int(result.registers[0])
@@ -301,25 +284,7 @@ class ModbusRegisterSwitch(ModbusCoilSwitch):
 
         return value
 
-    def _write_register(self, value):
+    async def _write_register(self, value):
         """Write holding register using the Modbus hub slave."""
-        try:
-            self._hub.write_register(self._slave, self._register, value)
-        except ConnectionException:
-            self._set_unavailable()
-            return
-
+        await self._hub.write_register(self._slave, self._register, value)
         self._available = True
-
-    def _set_unavailable(self):
-        """Set unavailable state and log it as an error."""
-        if not self._available:
-            return
-
-        _LOGGER.error(
-            "No response from hub %s, slave %s, register %s",
-            self._hub.name,
-            self._slave,
-            self._register,
-        )
-        self._available = False
